import serial.tools.list_ports
import serial
import time
from datetime import datetime

class CloudConnectionTester:
    def __init__(self, port=None):
        if not port:
            port = self._select_com_port()
        self.ser = serial.Serial(port, baudrate=115200, timeout=10)
        self.stats_reset()

    def stats_reset(self):
        self.total_cycles = 0
        self.successful_connections = 0
        self.failed_connections = 0
        self.successful_disconnections = 0

    def _select_com_port(self):
        while True:
            print("\nAvailable COM ports:")
            ports = serial.tools.list_ports.comports()
            for i, p in enumerate(ports):
                print(f"  [{i+1}] {p.device} - {p.description}")
                
            try:
                selection = int(input("Enter port number or 0 to refresh: "))
                if selection == 0:
                    continue
                return ports[selection-1].device
            except (ValueError, IndexError):
                print("Invalid selection! Please try again.")

    def _execute_command(self, command, success_indicator):
        try:
            print(f"\n>>> Sending command: {command}")
            self.ser.write(f"{command}\r\n".encode())
            start_time = time.time()
            response_buffer = ""
            
            while time.time() - start_time < 30:  # 30s timeout
                line = self.ser.readline().decode().strip()
                if line:
                    timestamp = datetime.now().strftime('%H:%M:%S.%f')[:-3]
                    print(f"[{timestamp}] {line}")
                    response_buffer += line + "\n"
                    
                    if "failed" in line.lower() or "rejected" in line.lower():
                        return False
                    if success_indicator in line:
                        return True
        
            return False
            
        except Exception as e:
            print(f"Serial communication error: {str(e)}")
            return False

    def connection_cycle(self):
        self.total_cycles += 1
        success = False
        
        # Connection phase
        connected = self._execute_command(
            "cloud connect", 
            "nrf_cloud_coap_transport: DTLS CID is active"
        )
        
        if connected:
            self.successful_connections += 1
            print("\033[92mConnection successful\033[0m")  # Green text
            time.sleep(5)  # Maintain connection briefly
            
            # Disconnection phase
            disconnected = self._execute_command(
                "cloud disconnect", 
                ""
            )
            
            if disconnected:
                self.successful_disconnections += 1
                print("\033[92mDisconnection successful\033[0m")
            else:
                print("\033[91mDisconnection failed!\033[0m")  # Red text
            
            return True
            
        else:
            self.failed_connections += 1
            print("\033[91mConnection failed!\033[0m")
            return False

    def generate_report(self):
        success_rate = (self.successful_connections/self.total_cycles)*100 if self.total_cycles else 0
        
        report = """
==========================================
       Connection Test Summary       
==========================================
Total cycles:          {}
Successful connections: {} ({}%)
Full success cycles:    {} ({}%)
Failed connections:     {}
Disconnect failures:    {}
==========================================
""".format(
    self.total_cycles,
    self.successful_connections,
    round(success_rate),
    self.successful_disconnections,
    round((self.successful_disconnections/self.total_cycles)*100) if self.total_cycles else 0,
    self.failed_connections,
    self.successful_connections - self.successful_disconnections,
)
            
        print(report)
        

if __name__ == "__main__":
    tester = CloudConnectionTester()
    
    try:
        while True:
            cycles = int(input("Number of test cycles (0 to quit): "))
            if cycles <= 0:
                break
            
            interval = int(input("Interval between cycles (seconds): "))
            
            for i in range(cycles):
                result = tester.connection_cycle()
                tester.generate_report()
                
                # Only wait if continuing tests and not last cycle
                if i < cycles-1 and interval > 0:  
                    print(f"\nWaiting {interval} seconds...")
                    time.sleep(interval)
                    
    except KeyboardInterrupt:
        print("\nTest interrupted by user!")
    
    finally:
        tester.generate_report()
